/* $Id: gvdevice_gtk.c,v 1.7 2005/07/11 18:35:56 ellson Exp $ $Revision: 1.7 $ */
/* vim:set shiftwidth=4 ts=8: */

/**********************************************************
*      This software is part of the graphviz package      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2004 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
**********************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <gtk/gtk.h>

#include <cairo-xlib.h>

#include "macros.h"
#include "const.h"
#include "types.h"

#include "gvplugin_device.h"

static gint delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
//    GVJ_t *job;

//    job = (GVJ_t *)data;
    gtk_main_quit();
    return FALSE;
}

static gboolean expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
    GVJ_t *job;
    cairo_t *cr;

    job = (GVJ_t *)data;
    cr = gdk_cairo_create(widget->window);

    gvevent_motion(job, job->pointer);

    job->surface = (void *)cr;
    job->external_surface = TRUE;
    job->width = widget->allocation.width;
    job->height = widget->allocation.height;
    gvevent_refresh(job);

    cairo_destroy(cr);

    return FALSE;
}

static gboolean motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
    GVJ_t *job = (GVJ_t *)data;

    job->pointer.x = event->x;
    job->pointer.y = event->y;
    gtk_widget_queue_draw_area(widget,
				widget->allocation.x,
				widget->allocation.y,
				widget->allocation.width,
				widget->allocation.height);
    return FALSE;
}


static void gtkgen_finalize(GVC_t *gvc)
{
    GVJ_t *job;
    GtkWidget *window, *darea;

    gtk_init (NULL, NULL);

    for (job = gvc->active_jobs; job; job = job->next_active) {

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	gtk_window_set_default_size(GTK_WINDOW(window), job->width, job->height);
	gtk_window_set_title(GTK_WINDOW(window), "graphviz:");

	darea = gtk_drawing_area_new();
	gtk_container_add (GTK_CONTAINER(window), darea);
	gtk_widget_add_events(darea,GDK_POINTER_MOTION_MASK);

	g_signal_connect(window, "delete-event", G_CALLBACK(delete_cb), (gpointer)job);
	g_signal_connect(darea, "expose-event", G_CALLBACK(expose_cb), (gpointer)job);
	g_signal_connect(darea, "motion-notify-event", G_CALLBACK(motion_cb), (gpointer)job);

	gtk_widget_show_all(window);
    }

    gtk_main();
}

gvdevice_engine_t gtkgen_device_engine = {
    gtkgen_finalize,
};

gvplugin_installed_t gvdevice_gtkgen_types[] = {
    {0, "gtk", 0, &gtkgen_device_engine, NULL},
    {0, NULL, 0, NULL, NULL}
};
